 /////////////////////////////////////////////////////////////////////////
//																		//
// Filename	: LA.cpp													//
// Project	: LA (Linear Algebra library)								//
// Author	: Iain Hutchison (iain.hutchison@st-annes.ox.ac.uk)			//
//																		//
//////////////////////////////////////////////////////////////////////////
//																		//
// Notes	:															//
//																		//
/////////////////////////////////////////////////////////////////////////

#include "LA.h"
#include <math.h>

 /////////////////////
//  class : Point4  //
/////////////////////

Point4::Point4() {
	x = y = z = 0.0f;
	w = 1.0f;
};

Point4::Point4(float nx,float ny,float nz,float nw) {
	x = nx;
	y = ny;
	z = nz;
	w = nw;
};

Point4::Point4(const Point4 &p) {
	x = p.x;
	y = p.y;
	z = p.z;
	w = p.w;
};

Point4::Point4(const Vector4 &v) {
	x = v.i;
	y = v.j;
	z = v.k;
	w = 1.0f;
};

Point4	Point4::operator=(Point4 p) {
	x = p.x;
	y = p.y;
	z = p.z;
	w = p.w;
	return *this;
};

int	Point4::operator==(Point4 p) {
	return ((x==p.x)&&(y==p.y)&&(z==p.z)&&(w==p.w));
};

int	Point4::operator!=(Point4 p) {
	return ((x!=p.x)||(y!=p.y)||(z!=p.z)||(w!=p.w));
};

Point4	Point4::operator+=(Vector4 v) {
	x += v.i;
	y += v.j;
	z += v.k;
	w += v.h;
	return *this;
};

Point4	Point4::operator-=(Vector4 v) {
	x -= v.i;
	y -= v.j;
	z -= v.k;
	w -= v.h;
	return *this;
};

Point4	Point4::operator+(Vector4 v) {
	Point4 p(*this);
	p += v;
	return p;
};

Point4	Point4::operator-(Vector4 v) {
	Point4 p(*this);
	p -= v;
	return p;
};

Vector4	Point4::operator-(Point4 p) {
	Vector4 v;
	v.i = x-p.x;
	v.i = y-p.y;
	v.i = z-p.z;
	return v;
};

float	Point4::operator[](int index) {
	index = index<0?0:(index>3?3:index);
	switch (index) {
		case 0 : return x; break;
		case 1 : return y; break;
		case 2 : return z; break;
		default : return w; break;
	};
};


 //////////////////////
//  class : Vector4  //
//////////////////////

Vector4::Vector4() {
	i = j = k = h = 0.0f;
};

Vector4::Vector4(float ni,float nj,float nk,float nh) {
	i = ni;
	j = nj;
	k = nk;
	h = nh;
};

Vector4::Vector4(const Vector4 &v) {
	i = v.i;
	j = v.j;
	k = v.k;
	h = v.h;
};

Vector4::Vector4(const Point4 &p) {
	i = p.x;
	j = p.y;
	k = p.z;
	h = 0.0f;
	if (p.z!=1.0f) {
		float z = 1.0f / p.z;
		i *= z;
		j *= z;
		k *= z;
	};
};

Vector4	Vector4::operator=(Vector4 v) {
	i = v.i;
	j = v.j;
	k = v.k;
	h = v.h;
	return *this;
};

int	Vector4::operator==(Vector4 v) {
	return ((i==v.i)&&(j==v.j)&&(k==v.k)&&(h==v.h));
};

int	Vector4::operator!=(Vector4 v) {
	return ((i!=v.i)||(j!=v.j)||(k!=v.k)||(h!=v.h));
};

Vector4	Vector4::operator+=(Vector4 v) {
	i += v.i;
	j += v.j;
	k += v.k;
	h += v.h;
	return *this;
};

Vector4	Vector4::operator-=(Vector4 v) {
	i -= v.i;
	j -= v.j;
	k -= v.k;
	h -= v.h;
	return *this;
};

Vector4	Vector4::operator*=(float f) {
	i *= f;
	j *= f;
	k *= f;
	h *= f;
	return *this;
};

Point4	Vector4::operator+(Point4 p) {
	Point4 p2(*this);
	p2 += p;
	return p2;
};

Vector4	Vector4::operator+(Vector4 v) {
	Vector4 v2(*this);
	v2 += v;
	return v2;
};

Vector4	Vector4::operator-(Vector4 v) {
	Vector4 v2(*this);
	v2 -= v;
	return v2;
};

Vector4	Vector4::operator*(float f) {
	Vector4 v(f*i,f*j,f*k,0.0f);
	return v;
};

Vector4	Vector4::operator-() {
	Vector4 v(-i,-j,-k,0.0f);
	return v;
};

float	Vector4::dotProduct(Vector4 v) {
	return (i*v.i+j*v.j+k*v.k+h*v.h);
};

float	Vector4::dotProduct(Point4 p) {
	return (i*p.x+j*p.y+k*p.z+h*p.w);
};

Vector4	Vector4::crossProduct(Vector4 v) {
	Vector4 r;
	r.i = j*v.k - k*v.j;
	r.j = k*v.i - i*v.k;
	r.k = i*v.j - j*v.i;
	r.h = 0.0f;
	return r;
};

float	Vector4::modulus() {
	return (float)sqrt(i*i+j*j+k*k+h*h);
};

float	Vector4::modulusSquared() {
	return (i*i+j*j+k*k+h*h);
};

void	Vector4::makeUnit() {
	float m = modulus();
	if (m>0)
		m = 1.0f / m;
	else
		m = 0.0f;
	i *= m;
	j *= m;
	k *= m;
	h *= m;
};

float&	Vector4::operator[](int index) {
	index = index<0?0:(index>3?3:index);
	switch (index) {
		case 0 : return i; break;
		case 1 : return j; break;
		case 2 : return k; break;
		default : return h; break;
	};
};


 ///////////////////////
//  class : Matrix44  //
///////////////////////

Matrix44::Matrix44() {
};

Matrix44::Matrix44(const Matrix44 &m) {
	rows[0] = m.rows[0];
	rows[1] = m.rows[1];
	rows[2] = m.rows[2];
	rows[3] = m.rows[3];
};
Matrix44	Matrix44::operator=(Matrix44 m) {
	rows[0] = m.rows[0];
	rows[1] = m.rows[1];
	rows[2] = m.rows[2];
	rows[3] = m.rows[3];
	return *this;
};

int	Matrix44::operator==(Matrix44 &m) {
	return ((rows[0]==m[0])&&(rows[1]==m[1])&&(rows[2]==m[2])&&(rows[3]==m[3]));
};

int	Matrix44::operator!=(Matrix44 &m) {
	return ((rows[0]!=m[0])||(rows[1]!=m[1])||(rows[2]!=m[2])||(rows[3]!=m[3]));
};

Matrix44	Matrix44::operator+=(Matrix44 &m) {
	rows[0] += m.rows[0];
	rows[1] += m.rows[1];
	rows[2] += m.rows[2];
	rows[3] += m.rows[3];
	return *this;
};

Matrix44	Matrix44::operator-=(Matrix44 &m) {
	rows[0] -= m.rows[0];
	rows[1] -= m.rows[1];
	rows[2] -= m.rows[2];
	rows[3] -= m.rows[3];
	return *this;
};

Matrix44	Matrix44::operator*=(Matrix44 &b) {
	Matrix44 a(*this);
	a.transpose();
	// can now calc matrix elements using dot products
	for (int i=0;i<4;i++) {
		rows[i][0] = a[i].dotProduct(b[0]);
		rows[i][1] = a[i].dotProduct(b[1]);
		rows[i][2] = a[i].dotProduct(b[2]);
		rows[i][3] = a[i].dotProduct(b[3]);
	};
	return *this;
};

Matrix44	Matrix44::operator*=(float f) {
	rows[0] *= f;
	rows[1] *= f;
	rows[2] *= f;
	rows[3] *= f;
	return *this;
};

Vector4&	Matrix44::operator[](int i) {
	i = i<0?0:(i>3?3:i);
	return rows[i];
};

Vector4 Matrix44::operator*(Vector4 v) {
	Vector4 r;
	r.i = rows[0].dotProduct(v);
	r.j = rows[1].dotProduct(v);
	r.k = rows[2].dotProduct(v);
	r.h = rows[3].dotProduct(v);
	return r;
};

Point4 Matrix44::operator*(Point4 p) {
	Point4 r;
	r.x = rows[0].dotProduct(p);
	r.y = rows[1].dotProduct(p);
	r.z = rows[2].dotProduct(p);
	r.w = rows[3].dotProduct(p);
	return r;
};

Matrix44	Matrix44::operator*(Matrix44 &b) {
	Matrix44 a(*this);
	a *= b;
	return a;
};

Matrix44	Matrix44::operator*(float f) {
	Matrix44 a(*this);
	a *= f;
	return a;
};

void	Matrix44::transpose() {
#define FSWAP(x,y,t) {t=x;x=y;y=t;}
	float t;
	for (int i=0;i<4;i++) {
		int j=i+1;
		while (j<4) {
			FSWAP(rows[i][j],rows[j][i],t);
			j++;
		}; // end while (j)
	}; // end for (i)
#undef FSWAP
};


 /////////////////////////////////////////////////////////////////////////
//															end of file	//
/////////////////////////////////////////////////////////////////////////
